import numpy as np

def normalize(dataset):
    if len(np.array(dataset).shape) > 1:
        min_vals = np.array(dataset).min(axis=0)
        max_vals = np.array(dataset).max(axis=0)
        for col in range(0, len(dataset[0])):
            for row in range(0, len(dataset)):
                if (max_vals[col] - min_vals[col]) != 0.0:
                    dataset[row][col] -=min_vals[col] 
                    dataset[row][col] /= (max_vals[col] - min_vals[col])
                else: dataset[row][col] = 1.0
        
        return (dataset, min_vals, max_vals)
    else:
        min_val = np.array(dataset).min()
        max_val = np.array(dataset).max()
        for index in range(0, len(dataset)):
            if (max_val - min_val) != 0.0:
                dataset[index] -= min_val
                dataset[index] /= (max_val - min_val)
            else: dataset[index] = 1.0
        return (dataset, [min_val], [max_val])
            

def denormalize(dataset, min_vals, max_vals):
    if len(np.array(dataset).shape) > 1:
        for col in range(0, len(dataset[0])):
            for row in range(0, len(dataset)):
                if (max_vals[col] - min_vals[col]) != 0.0:
                    dataset[row][col] *= (max_vals[col] - min_vals[col]) 
                    dataset[row][col] += min_vals[col]
                else: dataset[row][col] = min_vals[col]
    else:
        for index in range(0, len(dataset)):
            if (max_vals[0] - min_vals[0]) != 0.0:
                dataset[index] *= (max_vals[0] - min_vals[0]) 
                dataset[index] += min_vals[0]
            else: dataset[index] = min_vals[0]
            
    return dataset

def get_datasets(dataset, ratio):  
    shape_x = np.array(dataset).shape[0]
    shape_y = np.array(dataset).shape[1]
    
    train_start = 0
    train_end = train_start + int(shape_x * ratio)
        
    test_start = train_end + 1
    test_end = len(dataset)

    x_set = np.array(dataset)[:,2:shape_y + 2]
    y_set = np.array(dataset)[:,1]
       
    train_X = x_set[train_start:train_end:]
    train_Y = y_set[train_start:train_end:]

    test_X = x_set[test_start:test_end:]
    test_Y = y_set[test_start:test_end:]
    
    (train_X, train_min_vals_X, train_max_vals_X) = normalize(train_X)
    (train_Y, train_min_vals_Y, train_max_vals_Y) = normalize(train_Y)
    
    (test_X, test_min_vals_X, test_max_vals_X) = normalize(test_X)
    (test_Y, test_min_vals_Y, test_max_vals_Y) = normalize(test_Y)

    return (train_X, train_Y, test_X, test_Y, 
        train_min_vals_X, train_max_vals_X, train_min_vals_Y, train_max_vals_Y,
        test_min_vals_X, test_max_vals_X, test_min_vals_Y, test_max_vals_Y)

